% any.pl
% Dec 13, 2035
% Douglas Miles
%
% This file defines the basic ANY verb that has the four elements of 
% 
%  action_requires_states(Agent,StateRules,REQS),
%  action_removes_states(Agent,StateRules,REMS),
%  states_types(Agent,StateRules,TYPES),
%  action_adds_states(Agent,StateRules,ADDS),
% 

% :-swi_module(user). 
:- swi_module(any, []).



:- include(prologmud(mud_header)).

% :- register_module_type (mtCommand).
action_rules(_,_,_,_):-fail.

%=autodoc
%% action_rules( ?ARG1, ?ARG2, ?ARG3, ?ARG4) is semidet.
%
% Action Rules.
%




%=autodoc
%% to_verb_args( ?Act, ?List) is semidet.
%
% Converted To Verb Arguments.
%
to_verb_args(Act,List):-safe_univ(Act,List).



%=autodoc
%% action_adds_states( ?Agent, ?Action, ?A) is semidet.
%
% Action Adds States.
%
action_adds_states(Agent,Action,A):- action_rules(Agent,VERB,SENT,StateRules),to_verb_args(Action,[VERB|SENT]), member(+(A),StateRules).


%=autodoc
%% action_removes_states( ?Agent, ?Action, ?A) is semidet.
%
% Action Removes States.
%
action_removes_states(Agent,Action,A):- action_rules(Agent,VERB,SENT,StateRules),to_verb_args(Action,[VERB|SENT]), member(-(A),StateRules).


%=autodoc
%% action_requires_states( ?Agent, ?Action, ?A) is semidet.
%
% Action Requires States.
%
action_requires_states(Agent,Action,A):- action_rules(Agent,VERB,SENT,StateRules),to_verb_args(Action,[VERB|SENT]),member(?(A),StateRules).

:- op(200,fx,'?').
:- op(1050,fx,'==>').


%=autodoc
%% ==> ?IExplorer7 is semidet.
%
% ==>.
%
==>action_rules(Agent,actTestWield,[Obj],['?'(mudPossess(Agent,Obj)),'?'(isa(Obj,tUseAble)),-mudStowing(Agent,Obj),+mudWielding(Agent,Obj)]).
==>action_rules(Agent,actTestStow,[Obj],['?'(mudPossess(Agent,Obj)),'?'(isa(Obj,tStowAble)),'?'(genlPreds(Using,'mudControls')),
  - t(Using,Agent,Obj),+mudStowing(Agent,Obj)]).



%=autodoc
%% guess_verb_template( ?Action) is semidet.
%
% Guess Verb Template.
%
guess_verb_template(Action):-
       action_rules(_Agent,Verb,Args,ListA),
       once((to_verb_args(Action,[Verb|Args]),
            must(attempt_attribute_args(_AndOr,ftAskable,ListA)),
            must(term_variables(ListA,Vars)),
            must(attribs_to_atoms(Vars,Vars)))). 
         

% Use something
baseKB:agent_call_command(Agent,ACT) :-
   call((action_rules(Agent,VERB,SENT,_StateRules),safe_univ(ACT,[VERB|SENT]))),
   
      action_requires_states(Agent,[VERB|SENT],REQS),
      action_removes_states(Agent,[VERB|SENT],REMS),
      action_adds_states(Agent,[VERB|SENT],ADDS),
     call_update_charge(Agent,VERB),
     ((
         req1(REQS)) ->
         ((clr(REMS),
         ain(ADDS),
         call_update_charge(Agent,VERB)));	
%Nothing to use
      (add_cmdfailure(Agent,SENT))).

%=autodoc
%% agent_call_command( ?Agent, ?ACT) is semidet.
%
% Agent Call Command.
%


:- include(prologmud(mud_footer)).
